@@ -31,7 +31,6 @@ using System;
 using System.IO;
 using System.Text;
 using System.Security.Cryptography;
-using Mono.Security.Cryptography;
 using System.Collections.Generic;
 using System.Web;
 using System.Xml;
@@ -44,7 +43,7 @@ using Banshee.ServiceStack;
 using Banshee.Sources;
 
 using Lastfm;
-using Banshee.Lastfm.Radio;
+using Lastfm.Data;
 
 namespace Banshee.Lastfm.Audioscrobbler
 {
@@ -52,6 +51,8 @@ namespace Banshee.Lastfm.Audioscrobbler
     {
         internal class QueuedTrack
         {
+            private static DateTime epoch = DateTimeUtil.LocalUnixEpoch.ToUniversalTime ();
+
             public QueuedTrack (TrackInfo track, DateTime start_time)
             {
                 this.artist = track.ArtistName;
@@ -59,15 +60,18 @@ namespace Banshee.Lastfm.Audioscrobbler
                 this.title = track.TrackTitle;
                 this.track_number = (int) track.TrackNumber;
                 this.duration = (int) track.Duration.TotalSeconds;
-                this.start_time = DateTimeUtil.ToTimeT(start_time.ToLocalTime ());
+                // Idealy would use Hyena's DateTimeUtil, but it is broken since the "unix epoch" it uses is
+                // not UTC, so depending on whether jan 1 1970 was in day-light savings and whether the user's
+                // current timezone is in DLS, we'll be an hour off.
+                this.start_time = (long) (start_time.ToUniversalTime () - epoch).TotalSeconds;
                 // TODO
                 //this.musicbrainzid = track.MusicBrainzId;
-                
+
                 this.musicbrainzid = "";
-                
+
                 // set trackauth value, otherwise empty string is default
-                if (track is LastfmTrackInfo) {
-                    this.track_auth = (track as LastfmTrackInfo).TrackAuth;
+                if (track is ILastfmInfo) {
+                    this.track_auth = (track as ILastfmInfo).TrackAuth;
                 }
             }
 
@@ -88,31 +92,31 @@ namespace Banshee.Lastfm.Audioscrobbler
             public long StartTime {
                 get { return start_time; }
             }
-            
+
             public string Artist {
                 get { return artist; }
             }
-            
+
             public string Album {
                 get { return album; }
             }
-            
+
             public string Title {
                 get { return title; }
             }
-            
+
             public int TrackNumber {
                 get { return track_number; }
             }
-            
+
             public int Duration {
                 get { return duration; }
             }
-            
+
             public string MusicBrainzId {
                 get { return musicbrainzid; }
             }
-            
+
             public string TrackAuth {
                 get { return track_auth; }
             }
@@ -135,10 +139,10 @@ namespace Banshee.Lastfm.Audioscrobbler
 
         public Queue ()
         {
-            string xmlfilepath = Path.Combine (Banshee.Base.Paths.ExtensionCacheRoot, "last.fm");
+            string xmlfilepath = Path.Combine (Hyena.Paths.ExtensionCacheRoot, "last.fm");
             xml_path = Path.Combine (xmlfilepath, "audioscrobbler-queue.xml");
             queue = new List<QueuedTrack> ();
-            
+
             if (!Directory.Exists(xmlfilepath)) {
                 Directory.CreateDirectory (xmlfilepath);
             }
@@ -161,7 +165,7 @@ namespace Banshee.Lastfm.Audioscrobbler
 
             writer.WriteStartElement ("AudioscrobblerQueue");
             foreach (QueuedTrack track in queue) {
-                writer.WriteStartElement ("QueuedTrack");    
+                writer.WriteStartElement ("QueuedTrack");
                 writer.WriteElementString ("Artist", track.Artist);
                 writer.WriteElementString ("Album", track.Album);
                 writer.WriteElementString ("Title", track.Title);
@@ -188,7 +192,7 @@ namespace Banshee.Lastfm.Audioscrobbler
                 XmlNodeList nodes = doc.SelectNodes (query);
 
                 foreach (XmlNode node in nodes) {
-                    string artist = "";    
+                    string artist = "";
                     string album = "";
                     string title = "";
                     int track_number = 0;
@@ -220,7 +224,7 @@ namespace Banshee.Lastfm.Audioscrobbler
                     queue.Add (new QueuedTrack (artist, album, title, track_number, duration,
                         start_time, musicbrainzid, track_auth));
                 }
-            } catch { 
+            } catch {
             }
         }
 
@@ -234,14 +238,14 @@ namespace Banshee.Lastfm.Audioscrobbler
                 if (i == 49) break;
 
                 QueuedTrack track = (QueuedTrack) queue[i];
-                
+
                 string str_track_number = String.Empty;
                 if (track.TrackNumber != 0)
                     str_track_number = track.TrackNumber.ToString();
-                 
-                string source = "P"; /* chosen by user */   
+
+                string source = "P"; /* chosen by user */
                 if (track.TrackAuth.Length != 0) {
-                    // from last.fm 
+                    // from last.fm
                     source = "L" + track.TrackAuth;
                 }
 
@@ -250,13 +254,13 @@ namespace Banshee.Lastfm.Audioscrobbler
                     HttpUtility.UrlEncode (track.Artist),
                     HttpUtility.UrlEncode (track.Title),
                     track.StartTime.ToString (),
-                    source, 
-                    ""  /* rating: L/B/S */, 
+                    source,
+                    ""  /* rating: L/B/S */,
                     track.Duration.ToString (),
                     HttpUtility.UrlEncode (track.Album),
                     str_track_number,
                     track.MusicBrainzId,
-                    
+
                     i);
             }
 
@@ -268,7 +272,6 @@ namespace Banshee.Lastfm.Audioscrobbler
         {
             TrackInfo t = (track as TrackInfo);
             if (t != null) {
-                Log.DebugFormat ("Queued: {0}", t);
                 queue.Add (new QueuedTrack (t, started_at));
                 dirty = true;
                 RaiseTrackAdded (this, new EventArgs ());
